#pragma once

#include <vector>
#include <algorithm>
#include <cstdlib>
#include <cassert>
#include "base.h"
#include "force.h"

using namespace std;

class record_testcase_adversary : public target_testcase_adversary {
  public:
    record_testcase_adversary(vector<int> &b): b(b) {}
  protected:
    int target(int P, int B) {
      b.push_back(B);
      return B;
    }
  private:
    vector<int> &b;
};

class combine_two_adversary : public adversary {
  public:
    combine_two_adversary(): x(2), b(2) {}
    pair<int, int> testcase(int T, int t, int MAX_N) {
      int X = rand() % MAX_N + 1;
      if (t % 4 < 2) {
        x[t % 2] = X;
        b[t % 2].clear();
        testcaseAdversary = new record_testcase_adversary(b[t % 2]);
      } else {
        X = x[t % 2];
        vector<int> p(max(b[0].size(), b[1].size()));
        for (int i = 0; i < (int) b[0].size(); i++)
          p[i] = b[0][i];
        for (int i = 0; i < (int) b[1].size(); i++)
          if (i % 2 == 0 || i >= (int) b[0].size())
            p[i] = b[1][i];
        testcaseAdversary = new force_testcase_adversary(p);
      }
      return {MAX_N, X};
    }
  private:
    vector<int> x;
    vector<vector<int>> b;
};

class combine_three_adversary : public adversary {
  public:
    pair<int, int> testcase(int T, int t, int MAX_N, int G) {
      if (xs.empty()) {
        bool found = false;
        for (int i = 0; i < (int) b.size() - 1 && !found; i++)
          for (int j = i + 1; j < (int) b.size() - 1 && !found; j++)
            if (x[i] != x[j] && x[i] != x[b.size() - 1] && x[j] != x[b.size() - 1])
              found = G == 1 ? checkg1({i, j, (int) b.size() - 1}) : checkg2({i, j, (int) b.size() - 1});
      }
      int X = rand() % MAX_N + 1;
      if (xs.empty()) {
        x.push_back(X);
        b.emplace_back();
        testcaseAdversary = new record_testcase_adversary(b.back());
      } else {
        X = xs.back();
        xs.pop_back();
        testcaseAdversary = new force_testcase_adversary(p);
      }
      return {MAX_N, X};
    }
  private:
    vector<int> x, xs, p;
    vector<vector<int>> b;

    bool checkg1(vector<int> bc) {
      int l = 1;
      for (int c : bc)
        l = max(l, (int) b[c].size());
      vector<vector<int>> dp(l, vector<int>(2, -1));
      dp[0][0] = dp[0][1] = 0;
      for (int i = 0; i < l - 1; i++) {
        for (int b0 = 0; b0 < 2; b0++) {
          if (dp[i][b0] == -1)
            continue;
          for (int b1 = 0; b1 < 2; b1++) {
            bool f = false;
            for (int c : bc)
              f |= i + 1 < (int) b[c].size() && b[c][i] != b0 && b[c][i + 1] != b1;
            if (!f)
              dp[i + 1][b1] = b0;
          }
        }
        if (dp[i + 1][0] == -1 && dp[i + 1][1] == -1)
          return false;
      }
      int i = l - 1, b0 = 0;
      for (b0 = 0; b0 < 2 && dp[l - 1][b0] == -1; b0++);
      assert(b0 < 2);
      p.clear();
      while (i >= 0) {
        p.push_back(b0);
        b0 = dp[i][b0];
        i--;
      }
      reverse(p.begin(), p.end());
      for (int c : bc)
        xs.push_back(x[c]);
      return true;
    }

    bool checkg2(vector<int> bc) {
      int l = 1;
      for (int c : bc)
        l = max(l, (int) b[c].size());
      vector<vector<vector<int>>> dp(l, vector<vector<int>>(2, vector<int>(2, -1)));
      dp[0][0][0] = dp[0][0][1] = dp[0][1][0] = dp[0][1][1] = 0;
      for (int i = 0; i < l - 1; i++) {
        for (int b0 = 0; b0 < 2; b0++)
          for (int b1 = 0; b1 < 2; b1++) {
            if (dp[i][b0][b1] == -1)
              continue;
            for (int b2 = 0; b2 < 2; b2++) {
              bool f = false;
              for (int c : bc)
                f |= i + 1 < (int) b[c].size() && (i > 0 && b[c][i - 1] != b0 || b[c][i] != b1) && b[c][i + 1] != b2;
              if (!f)
                dp[i + 1][b1][b2] = b0;
            }
          }
        if (dp[i + 1][0][0] == -1 && dp[i + 1][0][1] == -1 && dp[i + 1][1][0] == -1 && dp[i + 1][1][1] == -1)
          return false;
      }
      int i = l - 1, b0 = 0, b1 = 0;
      for (b0 = 0; b0 < 2; b0++) {
        for (b1 = 0; b1 < 2 && dp[l - 1][b0][b1] == -1; b1++);
        if (b1 < 2 && dp[l - 1][b0][b1] != -1)
          break;
      }
      assert(b0 < 2);
      p.clear();
      while (i >= 0) {
        p.push_back(b1);
        b1 = dp[i][b0][b1];
        swap(b0, b1);
        i--;
      }
      reverse(p.begin(), p.end());
      for (int c : bc)
        xs.push_back(x[c]);
      return true;
    }
};