#pragma once

#include <vector>
#include <cstdlib>
#include "base.h"

using namespace std;

class force_testcase_adversary : public target_testcase_adversary {
  public:
    force_testcase_adversary(vector<int> p): p(p) {}
  protected:
    int target(int P, int B) {
      int t = p[P % p.size()];
      return t >= 0 && t < 2 ? t : B;
    }
  private:
    vector<int> p;
};

class force_adversary : public adversary {
  public:
    force_adversary(vector<vector<int>> p): p(p) {}
    pair<int, int> testcase(int T, int t, int MAX_N) {
      testcaseAdversary = new force_testcase_adversary(p[t % p.size()]);
      return {MAX_N, rand() % MAX_N + 1};
    }
  private:
    vector<vector<int>> p;
};

class adaptive_force_testcase_adversary : public force_testcase_adversary {
  public:
    adaptive_force_testcase_adversary(vector<int> p, int &b): force_testcase_adversary(p), b(b) {}
  protected:
    void written(int R) {
      b++;
    }
  private:
    int &b;
};

class adaptive_force_adversary : public adversary {
  public:
    adaptive_force_adversary(int s, vector<vector<int>> p): s(s), p(p), b(p.size()) {}
    pair<int, int> testcase(int T, int t, int MAX_N) {
      if (t == 0)
        for (int i = 0; i < (int) p.size(); i++)
          ri.push_back(i);
      if (ci == s * (int) ri.size()) {
        if (ci >= 2) {
          sort(ri.begin(), ri.end(), [&](int i, int j) { return b[i] > b[j]; });
          ri.resize(ci / 2);
        }
        ci = 0;
      }
      testcaseAdversary = new adaptive_force_testcase_adversary(p[ri[ci % (int) ri.size()]], b[ri[ci % (int) ri.size()]]);
      ci++;
      return {MAX_N, rand() % MAX_N + 1};
    }
  private:
    int s, ci = 0;
    vector<int> ri, b;
    vector<vector<int>> p;
};

class daynamic_force_testcase_adversary : public force_testcase_adversary {
  public:
    daynamic_force_testcase_adversary(vector<int> p, vector<int> &b): force_testcase_adversary(p), b(b) {}
  protected:
    void written(int P, int B, int T, int R) {
      b.push_back(B);
    }
  private:
    vector<int> &b;
};

class dynamic_force_adversary : public adversary {
  public:
    dynamic_force_adversary(int st, int n, int s): st(st), n(n), s(s) {}
    pair<int, int> testcase(int T, int t, int MAX_N) {
      if (t < st) {
        b.emplace_back();
        // vector<int> cp;
        // for (int i = 0; i < 200; i++)
        //   cp.push_back(rand() & 1);
        testcaseAdversary = new daynamic_force_testcase_adversary({0}, b.back());
      } else {
        if (t == st) {
          findPattern();
          for (int i = 0; i < (int) p.size(); i++)
            ri.push_back(i), c.push_back(0);
        }
        if (ci == s * (int) ri.size()) {
          if (ci >= 2) {
            sort(ri.begin(), ri.end(), [&](int i, int j) { return c[i] > c[j]; });
            ri.resize(ci / 2);
          }
          ci = 0;
        }
        testcaseAdversary = new adaptive_force_testcase_adversary(p[ri[ci % (int) ri.size()]], c[ri[ci % (int) ri.size()]]);
        ci++;
      }
      return {MAX_N, rand() % MAX_N + 1};
    }
  private:
    int st, n, s, ci = 0;
    vector<int> ri, c;
    vector<vector<int>> p, b;

    void findPattern() {
      vector<tuple<int, int, int>> pp;
      for (int l = 1; l < 8; l++) {
        for (int cp = 0; cp < (1 << l); cp++) {
          bool m = true;
          for (int d = 1; d < l; d++)
            if (l % d == 0 && (cp & ((1 << (l - d)) - 1)) == (cp >> d))
              m = false;
          if (!m)
            continue;
          int r = 0;
          for (vector<int> &cb : b) {
            int lm = -1, cbp = 0;
            for (int i = 0; i < (int) cb.size(); i++) {
              cbp = (cbp >> 1) | (cb[i] << (l - 1));
              int d = cp ^ cbp;
              if (i >= lm + l && (d & (d >> 1)) == 0)
                r++, lm = i;
            }
          }
          r *= l;
          pp.emplace_back(r, cp, l);
        }
      }
      sort(pp.rbegin(), pp.rend());
      for (int i = 0; i < (int) pp.size() && i < n; i++) {
        p.emplace_back();
        for (int j = 0; j < get<2>(pp[i]); j++)
          p.back().push_back((get<1>(pp[i]) >> j) & 1);
      }
    }
};