#pragma once

#include "base.h"

class fusion_force_testcase_adversary : public testcase_adversary_unison {
  public:
    fusion_force_testcase_adversary(int num_instances, vector<int> target) :
      testcase_adversary_unison(num_instances), flipped(num_instances, false), target(target), counter(-1) {
        assert(num_instances == 2);
      }
    virtual ~fusion_force_testcase_adversary() {}

  protected:
    vector<char> flipped;
    vector<int> target;
    int counter;

    vector<int> step(vector<int> S) {
      ++counter;
      assert(S.size() == 2);
      assert(flipped.size() == 2);

      if(S[0] == S[1]) {
        flipped[0] = flipped[1] = false;
        return S;
      }

      if(flipped[0]) {
        flipped = {false, true};
        return {S[0], 1 - S[1]};
      }

      if(flipped[1]) {
        flipped = {true, false};
        return {1 - S[0], S[1]};
      }

      int b = target[counter % target.size()];
      flipped = {S[0] != b, S[1] != b};
      return {b, b};
    }
};

void print_hex(vector<int> target) {
        int acc = 0;
      int counter = 0;

      fprintf(stderr, "%d:", (int) target.size());

      for(int b : target) {
        acc = (acc << 1) | b;
        ++counter;
        if(counter % 4 == 0) {fprintf(stderr, "%x", acc); acc=0;}
      }

      if(counter % 4) fprintf(stderr, "%x", acc);
}

class fusion_rotate_adversary : public adversary {
  public:
    fusion_rotate_adversary(int num_instances) : counter(0), num_instances(num_instances) {
      assert(num_instances == 2);
      for(int i = 0; i < period + 1; ++i) X.push_back(rand());
      for(int i = 0; i < 1000; ++i) target.push_back(rand() % 2);
    }

    virtual pair<int, int> testcase(int T, int t, int MAX_N) {
      ++counter;
      int idx = (counter % 2 == 0 ? period : (counter / 2) % period);
      int x = X[idx];
      if(idx) X[idx] = rand();
      return {MAX_N, x % MAX_N + 1};
    }

    virtual void newRun() {
      assert(testcaseAdversary == 0);
      testcaseAdversary =
        new record_output_testcase_adversary(new fusion_force_testcase_adversary(num_instances, target));
    }

    virtual void testcaseEnd() {
      if(testcaseAdversary) {
        record_output_testcase_adversary *adv = dynamic_cast<record_output_testcase_adversary *>(testcaseAdversary);
        assert(adv->getOutput(0) == adv->getOutput(1));
        target = adv->getOutput(0);
      }
      /*fprintf(stderr, "[target is ");
      print_hex(target);
      fprintf(stderr, "]");*/

      adversary::testcaseEnd();
    }

  private:
    int num_instances = 2;
    static constexpr int period = 2;
    vector<int> X;
    vector<int> target;
    int counter;
};