#pragma once

#include "base.h"
#include <unordered_set>
#include <string>

class fusion_scatter_testcase_adversary : public testcase_adversary_unison {
  public:
    fusion_scatter_testcase_adversary(int num_instances, vector<vector<int>> &stat) :
      testcase_adversary_unison(num_instances), flipped(num_instances, false), stat(stat), new_stat(stat), counter(-1) {
        assert(num_instances == 2);
      }

  protected:
    vector<char> flipped;
    vector<vector<int>> stat;
    vector<vector<int>> &new_stat;
    int counter;

    vector<int> myStep(vector<int> S) {
      assert(S.size() == 2);
      assert(flipped.size() == 2);

      if(S[0] == S[1]) {
        flipped[0] = flipped[1] = false;
        return S;
      }

      if(flipped[0]) {
        flipped = {false, true};
        return {S[0], 1 - S[1]};
      }

      if(flipped[1]) {
        flipped = {true, false};
        return {1 - S[0], S[1]};
      }

      int b = rand() % 2;
      if(stat[counter][b] > stat[counter][1 - b] ? 1 - b : b); // sic!
      flipped = {S[0] != b, S[1] != b};
      return {b, b};
    }

    vector<int> step(vector<int> S) {
      ++counter;
      S = myStep(S);
      for(int x : S) ++new_stat[counter][x];
      return S;
    }
};

class fusion_scatter_adversary : public adversary {
  public:
    fusion_scatter_adversary(int num_instances) :
      num_instances(num_instances), counter(-1), X(num_instances), stat(1337, vector<int>(2, 0)) {
        assert(num_instances == 2);
        X[0] = rand();
        X[1] = rand();
      }

    virtual pair<int, int> testcase(int T, int t, int MAX_N) {
      ++counter;
      return {MAX_N, 1 + X[counter % 2] % MAX_N};
    }

    virtual void newRun() {
      assert(testcaseAdversary == 0);
      testcaseAdversary = new record_output_testcase_adversary(new fusion_scatter_testcase_adversary(num_instances, stat));
    }

    virtual void testcaseEnd() {
      auto x = dynamic_cast<record_output_testcase_adversary *>(testcaseAdversary)->getOutput(0);
      string s; for(int i : x) s.push_back('0' + i);
      /*fprintf(stderr, "[%s]                                                                                               ", s.c_str());*/
      S.insert(s);
      //fprintf(stderr, "(%d)", (int) S.size());
      adversary::testcaseEnd();
    }

  private:
    int num_instances;
    int counter;
    vector<int> X;
    unordered_set<string> S;
    vector<vector<int>> stat;
};