#include <stubmanagerframework.h>
#include <bits/stdc++.h>
using namespace std;

const int QUESTION = 942318468,
            ANSWER = 539883183;

//#define DEBUG

#ifdef DEBUG
#define TOO_MANY_QUERIES "Too many queries"
#define INVALID_QUERY "Invalid call to ask"
#define WRONG_ANSWER "Wrong answer"
#define NOT_ENOUGH_QUERIES "Answer not unique yet"
#define NO_ANSWER "No answer"
#define CORRECT "Correct"
#define SECURITY_VIOLATION "Security violation!"
#else
#define TOO_MANY_QUERIES "Not correct"
#define INVALID_QUERY "Not correct"
#define WRONG_ANSWER "Not correct"
#define NOT_ENOUGH_QUERIES "Not correct"
#define NO_ANSWER "Not correct"
#define CORRECT "Correct"
#define SECURITY_VIOLATION "Security violation!"
#endif

//computes the inversion count on an array containing (1, ..., n) in some permutation in O(n log n) using a Fenwick Tree.
int inversion_count(const vector<int> &in) {
    int size = in.size()+1;
    vector<int>v(size);

    int sum = 0;
    for(int i=in.size()-1;i>=0;i--){
        int a = in[i];
        while(a){
            sum+=v[a];
            a-=a&-a;
        }
        a = in[i];
        while(a<size){
            v[a]++;
            a+=a&-a;
        }
    }
    return sum;
}

int read_int_or_fail()
{
    int i;
    if (fscanf(fcommin, "%d", &i) != 1)
        result(0.0, SECURITY_VIOLATION);
    return i;
}

void check()
{
    message_on_shutdown = true;

    int n, max_q, q;
    char tmp[128];
    assert(fscanf(fin, "%d %d %s\n", &n, &max_q, tmp) == 3);
    q = max_q;
    string input_type = tmp;
    vector<int> solution(n);
    if(input_type == "verbatim") {
        for(auto &x : solution)
            assert(fscanf(fin, "%d", &x) == 1);
    }
    else if(input_type == "random") {
        int seed;
        assert(fscanf(fin, "%d", &seed) == 1);
        mt19937 rengine(seed);
        iota(solution.begin(), solution.end(), 1);
        shuffle(solution.begin(), solution.end(), rengine);
    }
    else
        assert(false);

    vector<int> solution_inv(n);
    for(int i = 0; i < n; ++i)
        solution_inv[solution[i]-1] = i;

    fprintf(fcommout, "%d\n", n);
    fflush(fcommout);

    for (;;)
    {
        int type;
        int fscanf_return = fscanf(fcommin, "%d", &type);

        if (fscanf_return == EOF)
            break;
        else if (fscanf_return != 1)
            result(0.0, SECURITY_VIOLATION);

        switch (type)
        {
            case QUESTION: {
                vector<int> order(n);
                if(read_int_or_fail() != n)
                    result(0.0, INVALID_QUERY);
                for(auto &x : order)
                    x = read_int_or_fail();

#ifdef DEBUG
                if (--q < -500)
                    result(0.0, TOO_MANY_QUERIES);
#else
                if (--q < 0)
                    result(0.0, TOO_MANY_QUERIES);
#endif
                set<int> seen;
                for(auto &x : order) {
                    if(x <= 0 || x > n)
                        result(0, INVALID_QUERY);
                    if(seen.count(x))
                        result(0, INVALID_QUERY);
                    seen.insert(x);
                }

                vector<int> shuffled_order(n);
                for(int i = 0; i < n; ++i) {
                    shuffled_order[i] = solution_inv[order[i]-1]+1;
                }

                fprintf(fcommout, "%d\n", inversion_count(shuffled_order));
                fflush(fcommout);
                break; }

            case ANSWER: {
                vector<int> answer(n);
                if(read_int_or_fail() != n)
                    result(0.0, INVALID_QUERY);
                for(int i = 0; i < n; ++i)
                    answer[i] = read_int_or_fail();
                if(answer != solution) {
                    /*string res = "WA:";
                    for(auto &x : answer) res += " " + to_string(x);
                    res += " instead of";
                    for(auto &x : solution) res += " " + to_string(x);
                    result(0, res.c_str());*/
                    result(0, WRONG_ANSWER);
                }
#ifdef DEBUG
                if (q < 0)
                    result(0.0, TOO_MANY_QUERIES " with %d queries too many", -q);
                result(1.0, CORRECT " with %d queries remaining", q);
#else
                result(1.0, CORRECT);
#endif
                }
            default:
                result(0.0, SECURITY_VIOLATION);
        }
    }
    result(0.0, NO_ANSWER);
}
